use std::{future::Future, pin::Pin, sync::Arc, task::Poll};

use keyfork_derive_util::DerivationPath;
use keyfork_mnemonic_util::Mnemonic;
use thiserror::Error;
use tower::Service;

#[derive(Clone, Debug)]
pub struct Keyforkd {
    mnemonic: Arc<Mnemonic>,
}

// Currently, this can't be instantiated, therefore it is a never-type
#[derive(Debug, Error)]
pub enum DerivationError {}

impl Keyforkd {
    pub fn new(mnemonic: Mnemonic) -> Self {
        Self {
            mnemonic: Arc::new(mnemonic),
        }
    }
}

impl Service<DerivationPath> for Keyforkd {
    type Response = Vec<u8>;

    type Error = DerivationError;

    type Future = Pin<Box<dyn Future<Output = Result<Self::Response, Self::Error>> + Send>>;

    fn poll_ready(
        &mut self,
        _cx: &mut std::task::Context<'_>,
    ) -> std::task::Poll<Result<(), Self::Error>> {
        Poll::Ready(Ok(()))
    }

    #[cfg_attr(feature = "tracing", tracing::instrument(skip(self)))]
    fn call(&mut self, req: DerivationPath) -> Self::Future {
        dbg!(&req, &self.mnemonic);
        Box::pin(async { Ok(vec![]) })
    }
}
